@@ -1,13 +1,32 @@
 """This module contains id query."""
 from __future__ import annotations
-from typing import (Union, TypeVar, Generator, Optional, Any, Generic,
-                    overload, cast, TYPE_CHECKING)
+from typing import (Iterator, Union, TypeVar, Generator, Optional, Any,
+                    Generic, overload, cast)
 from bson import ObjectId
 from jsonclasses.exceptions import ObjectNotFoundException
 from inflection import camelize
+from pymongo.cursor import Cursor
 from .decoder import Decoder
 from .connection import Connection
-T = TypeVar('T')
+from .pymongo_object import PymongoObject
+T = TypeVar('T', bound=PymongoObject)
+
+
+def _update_cases(filter: Optional[dict[str, Any]],
+                  camelize_db_keys: bool) -> dict[str, Any]:
+    if filter is None:
+        return {}
+    if filter.get('id'):
+        filter['_id'] = ObjectId(filter['id'])
+        del filter['id']
+    if not camelize_db_keys:
+        return filter
+    if filter is None:
+        return None
+    retval: dict[str, Any] = {}
+    for key, value in filter.items():
+        retval[camelize(key, False)] = value
+    return retval
 
 
 class IDQuery(Generic[T]):
@@ -82,26 +101,12 @@ class ListQuery(BaseQuery, Generic[T]):
                  cls: type[T],
                  filter: Optional[dict[str, Any]] = None) -> None:
         super().__init__(cls=cls)
-        if filter is not None and filter.get('id'):
-            filter['_id'] = ObjectId(filter['id'])
-            del filter['id']
-        self.filter = self._update_cases(filter)
+        self.filter = _update_cases(filter, cls.dbconf.camelize_db_keys)
 
     def __call__(self, filter: Optional[dict[str, Any]] = None) -> ListQuery:
-        self.filter = self._update_cases(filter)
+        self.filter = _update_cases(filter, self.cls.dbconf.camelize_db_keys)
         return self
 
-    def _update_cases(self,
-                      d: Optional[dict[str, Any]]) -> Optional[dict[str, Any]]:
-        if not self.cls.dbconf.camelize_db_keys:
-            return d
-        if d is None:
-            return None
-        retval: dict[str, Any] = {}
-        for key, value in d.items():
-            retval[camelize(key, False)] = value
-        return retval
-
     def exec(self) -> list[T]:
         return super()._exec()
 
@@ -183,3 +188,49 @@ class OptionalSingleQuery(SingleQuery, Generic[T]):
         if len(results) < 1:
             return None
         return results[0]
+
+
+class ExistQuery(Generic[T]):
+
+    def __init__(self,
+                 cls: type[T],
+                 filter: Optional[dict[str, Any]] = None) -> None:
+        self.cls = cls
+        self.filter = _update_cases(filter, cls.dbconf.camelize_db_keys)
+
+    def exec(self) -> bool:
+        result = Connection.get_collection(self.cls).count_documents(
+                self.filter, limit=1)
+        return False if result == 0 else True
+
+    def __await__(self) -> Generator[None, None, bool]:
+        yield
+        return self.exec()
+
+
+class QueryIterator(Generic[T]):
+
+    def __init__(self, cls: type[T], cursor: Cursor):
+        self.cls = cls
+        self.cursor = cursor
+
+    def __iter__(self):
+        return self
+
+    def __next__(self) -> T:
+        value = self.cursor.__next__()
+        return Decoder().decode_root(value, self.cls)
+
+
+class IterateQuery(Generic[T]):
+
+    def __init__(self,
+                 cls: type[T],
+                 filter: Optional[dict[str, Any]] = None) -> None:
+        self.cls = cls
+        self.filter = _update_cases(filter, cls.dbconf.camelize_db_keys)
+
+    def exec(self) -> Iterator[T]:
+        coll = Connection.from_class(self.cls).get_collection(self.cls)
+        cursor = coll.find(self.filter)
+        return QueryIterator(cls=self.cls, cursor=cursor)
